@@ -36,7 +36,7 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: olsrd_plugin.c,v 1.17 2007/07/15 19:29:38 bernd67 Exp $
+ * $Id: olsrd_plugin.c,v 1.20 2007/09/17 21:57:05 bernd67 Exp $
  */
 
 /*
@@ -64,8 +64,8 @@ struct allowed_net *allowed_nets = NULL;
 static void my_init(void) __attribute__ ((constructor));
 static void my_fini(void) __attribute__ ((destructor));
 
-static int add_plugin_ipnet(const char *value, void *data);
-static int add_plugin_ipaddr(const char *value, void *data);
+static int add_plugin_ipnet(const char *value, void *data, set_plugin_parameter_addon);
+static int add_plugin_ipaddr(const char *value, void *data, set_plugin_parameter_addon);
 
 static int insert_plugin_ipnet(const char *sz_net, const char *sz_mask, struct allowed_net **allowed_nets);
 
@@ -106,7 +106,7 @@ static const struct olsrd_plugin_parameters plugin_parameters[] = {
     { .name = "port",   .set_plugin_parameter = &set_plugin_port,      .data = &http_port },
     { .name = "host",   .set_plugin_parameter = &add_plugin_ipaddr,    .data = &allowed_nets },
     { .name = "net",    .set_plugin_parameter = &add_plugin_ipnet,     .data = &allowed_nets },
-    { .name = "resolve",.set_plugin_parameter = &set_boolean,          .data = &resolve_ip_addresses },
+    { .name = "resolve",.set_plugin_parameter = &set_plugin_boolean,          .data = &resolve_ip_addresses },
 };
 
 void olsrd_get_plugin_parameters(const struct olsrd_plugin_parameters **params, int *size)
@@ -140,7 +140,7 @@ static int insert_plugin_ipnet(const char *sz_net, const char *sz_mask, struct a
     return 0;
 }
 
-static int add_plugin_ipnet(const char *value, void *data)
+static int add_plugin_ipnet(const char *value, void *data, set_plugin_parameter_addon addon __attribute__((unused)))
 {
     char sz_net[100], sz_mask[100]; /* IPv6 in the future */
 
@@ -151,7 +151,7 @@ static int add_plugin_ipnet(const char *value, void *data)
     return insert_plugin_ipnet(sz_net, sz_mask, data);
 }
 
-static int add_plugin_ipaddr(const char *value, void *data)
+static int add_plugin_ipaddr(const char *value, void *data, set_plugin_parameter_addon addon __attribute__((unused)))
 {
     return insert_plugin_ipnet(value, "255.255.255.255", data);
 }
