@@ -32,6 +32,7 @@
 #include "log.h"
 #include "defs.h"
 #include "local_hna_set.h"
+#include "routing_table.h"
 
 #ifdef USE_UNIX_DOMAIN_SOCKET
 #include <sys/un.h>
@@ -46,9 +47,11 @@
 #define BUFSIZE 1024
 
 #define STATUS_CONNECTED 1
+#define OPTION_EXPORT 1
 
 static struct {
-  char status; // TODO: internal status
+  char status; // internal status
+  char options; // internal options
   int sock; // Socket to zebra...
   char redistribute[ZEBRA_ROUTE_MAX];
   char distance;
@@ -58,32 +61,33 @@ static struct {
 
 
 /* prototypes intern */
-static char *try_read (ssize_t *);
-static char* zebra_route_packet (struct ipv4_route r, ssize_t *);
-static int parse_interface_add (char *, size_t);
-static int parse_interface_delete (char *, size_t);
-static int parse_interface_up (char *, size_t);
-static int parse_interface_down (char *, size_t);
-static int parse_interface_address_add (char *, size_t);
-static int parse_interface_address_delete (char *, size_t);
-static int parse_ipv4_route (char *, size_t, struct ipv4_route *);
-static int ipv4_route_add (char *, size_t);
-static int ipv4_route_delete (char *, size_t);
-static int parse_ipv6_route_add (char*, size_t);
-static int zebra_reconnect (void);
-static int zebra_connect (void);
-static int add_v4_route_status (struct ipv4_route r);
-static int del_v4_route_status (struct ipv4_route r);
+static unsigned char *try_read (ssize_t *);
+static unsigned char* zebra_route_packet (struct ipv4_route r, ssize_t *);
+static int parse_interface_add (unsigned char *, size_t);
+static int parse_interface_delete (unsigned char *, size_t);
+static int parse_interface_up (unsigned char *, size_t);
+static int parse_interface_down (unsigned char *, size_t);
+static int parse_interface_address_add (unsigned char *, size_t);
+static int parse_interface_address_delete (unsigned char *, size_t);
+static int parse_ipv4_route (unsigned char *, size_t, struct ipv4_route *);
+static int ipv4_route_add (unsigned char *, size_t);
+static int ipv4_route_delete (unsigned char *, size_t);
+static int parse_ipv6_route_add (unsigned char*, size_t);
+static void zebra_reconnect (void);
+static void zebra_connect (void);
+
 static uint32_t prefixlentomask (uint8_t);
 static void free_ipv4_route (struct ipv4_route);
+/* 
 static void update_olsr_zebra_routes (struct ipv4_route*, struct ipv4_route*);
 static struct ipv4_route *zebra_create_ipv4_route_table_entry (uint32_t,
 							       uint32_t,
 							       uint32_t);
 static struct ipv4_route *zebra_create_ipv4_route_table (void);
 static void zebra_free_ipv4_route_table (struct ipv4_route*);
-static uint8_t masktoprefixlen (uint32_t);
+*/
 
+/*static uint8_t masktoprefixlen (uint32_t);*/
 
 
 #ifdef MY_DEBUG
@@ -147,122 +151,76 @@ static void dump_ipv4_route (struct ipv4_route r, char *c) {
 void *my_realloc (void *buf, size_t s, const char *c) {
   buf = realloc (buf, s);
   if (!buf) {
-    OLSR_PRINTF (1, "OUT OF MEMORY: %s\n", strerror(errno));
+    OLSR_PRINTF (1, "(QUAGGA) OUT OF MEMORY: %s\n", strerror(errno));
     olsr_syslog(OLSR_LOG_ERR, "olsrd: out of memory!: %m\n");
     olsr_exit(c, EXIT_FAILURE);
   }
   return buf;
 }
 
-int init_zebra () {
-  if (zebra_connect() < 0 || !(zebra.status&STATUS_CONNECTED)) {
-    olsr_exit ("AIIIII, could not connect to zebra! is zebra running?", 
+void init_zebra (void) {
+  zebra_connect();
+  if (!zebra.status&STATUS_CONNECTED)
+    olsr_exit ("(QUAGGA) AIIIII, could not connect to zebra! is zebra running?", 
 	       EXIT_FAILURE);
+}
+
+void zebra_cleanup (void) {
+  int i;
+  struct rt_entry *tmp;
+  
+  if (zebra.options & OPTION_EXPORT) {
+    OLSR_FOR_ALL_RT_ENTRIES(tmp) {
+      zebra_del_olsr_v4_route(tmp);
+    } OLSR_FOR_ALL_RT_ENTRIES_END(tmp);
   }
+
+  for (i = 0; i < ZEBRA_ROUTE_MAX; i++)
+    if (zebra.redistribute[i]) zebra_disable_redistribute(i + 1);
 }
 
 
-static int zebra_reconnect (void) {
-  struct ipv4_route *tmp;
+static void zebra_reconnect (void) {
+  struct rt_entry *tmp;
   int i;
 
-  if (zebra_connect() < 0 || !(zebra.status&STATUS_CONNECTED)) 
-    // log: zebra-reconnect failed
-    ;
-  for (i = 0; ZEBRA_ROUTE_MAX - 1; i++)
+  zebra_connect();
+  if (!zebra.status & STATUS_CONNECTED) return; // try again next time
+
+  if (zebra.options & OPTION_EXPORT) {
+    OLSR_FOR_ALL_RT_ENTRIES(tmp) {
+      zebra_add_olsr_v4_route (tmp);
+    } OLSR_FOR_ALL_RT_ENTRIES_END(tmp);
+  }  
+
+  for (i = 0; i < ZEBRA_ROUTE_MAX; i++)
     if (zebra.redistribute[i]) zebra_redistribute(i + 1);
-
-  for (tmp = zebra.v4_rt; tmp; tmp = tmp->next)
-    zebra_add_v4_route(*tmp);
+  /* Zebra sends us all routes of type it knows after 
+     zebra_redistribute(type) */
 }
 
 
-static int add_v4_route_status (struct ipv4_route r) {
-
-  struct ipv4_route *tmp = olsr_malloc (sizeof r, "quagga_v4_route_status");
-  memcpy (tmp, &r, sizeof r);
-
-  if (r.message & ZAPI_MESSAGE_NEXTHOP) {
-    tmp->nexthops = olsr_malloc (r.nh_count * sizeof tmp->nexthops, 
-				 "quagga_v4_route_status");
-    memcpy (tmp->nexthops, &r.nexthops, sizeof *r.nexthops);
-  }
-
-  if (r.message & ZAPI_MESSAGE_IFINDEX) {
-    tmp->index = olsr_malloc (r.ind_num * sizeof *tmp->index, 
-			      "quagga_v4_route_status");
-    memcpy (tmp->index, &r.index, r.ind_num * sizeof *tmp->index);
-  }
-
-  tmp->next = zebra.v4_rt;
-  zebra.v4_rt = tmp;
-
-  return 0;
-
-}
-
-
-static int cmp_v4_route (struct ipv4_route a, struct ipv4_route b) {
-  if (a.type != b.type) return 1;
-  if (a.flags != b.flags) return 1;
-  if (a.message != b.message) return 1;
-  if (a.prefixlen != b.prefixlen) return 1;
-  if (a.message & ZAPI_MESSAGE_NEXTHOP) {
-    if (a.nh_count != b.nh_count) return 1;
-    if (memcmp (a.nexthops, b.nexthops, a.nh_count * sizeof *b.nexthops)) 
-      return 1;
-  }
-  if (a.message & ZAPI_MESSAGE_IFINDEX) {
-    if (a.ind_num != b.ind_num) return 1;
-    if (memcpy (a.index, b.index, a.ind_num * sizeof *a.index)) return 1;
-  }
-  if (a.message & ZAPI_MESSAGE_DISTANCE) 
-    if (a.distance != b.distance) return 1;
-  if (a.message & ZAPI_MESSAGE_METRIC)
-    if (a.metric != b.metric) return 1;
-  return 0;
-}
-
-static int del_v4_route_status (struct ipv4_route r) {
-
-  struct ipv4_route *tmp, *prv = 0;
-
-  for (tmp = zebra.v4_rt; tmp; tmp = tmp->next) {
-    if (!cmp_v4_route(*tmp, r)) {
-      if (prv) prv->next = tmp->next;
-
-      free_ipv4_route(*tmp);
-      free (tmp);
-
-      return 0;
-
-    }
-    prv = tmp;
-  }
-
-  return 1;
-}
-
 
 /* Connect to the zebra-daemon, returns a socket */
-static int zebra_connect (void) {
+static void zebra_connect (void) {
+  
+  int ret;
 
 #ifndef USE_UNIX_DOMAIN_SOCKET
   struct sockaddr_in i;
-  close (zebra.sock);
+  if (close (zebra.sock) < 0) olsr_exit ("(QUAGGA) Could not close socket!", EXIT_FAILURE);
+  
 
   zebra.sock = socket (AF_INET,SOCK_STREAM, 0);
 #else
   struct sockaddr_un i;
-  close (zebra.sock);
+  if (close (zebra.sock) < 0) olsr_exit ("(QUAGGA) Could not close socket!", EXIT_FAILURE);
 
   zebra.sock = socket (AF_UNIX,SOCK_STREAM, 0);
 #endif
 
-  int ret;
-
   if (zebra.sock <0 )
-    olsr_exit("could not create socket!", EXIT_FAILURE);
+    olsr_exit("(QUAGGA) Could not create socket!", EXIT_FAILURE);
   
   memset (&i, 0, sizeof i);
 #ifndef USE_UNIX_DOMAIN_SOCKET
@@ -275,18 +233,16 @@ static int zebra_connect (void) {
 #endif
 
   ret = connect (zebra.sock, (struct sockaddr *)&i, sizeof i);
-  if  (ret < 0) {
-    close (zebra.sock);
-  }
+  if  (ret < 0) zebra.status &= ~STATUS_CONNECTED;
   else zebra.status |= STATUS_CONNECTED;
-  return zebra.sock;
 }
 
     
 /* Sends a command to zebra, command is 
    the command defined in zebra.h, options is the packet-payload, 
    optlen the length, of the payload */
-char zebra_send_command (unsigned char command, char * options, int optlen) {
+unsigned char zebra_send_command (unsigned char command, 
+                                  unsigned char *options, int optlen) {
 
 #ifdef ZEBRA_HEADER_MARKER
   char *p = olsr_malloc (optlen + 6, "zebra_send_command");
@@ -297,10 +253,11 @@ char zebra_send_command (unsigned char command, char * options, int optlen) {
   char *p = olsr_malloc (optlen + 3, "zebra_send_command");
   uint16_t length = optlen + 3;  // length of option + command + packet_length
 #endif
-  
+  char *pnt = p;
   int ret;
   
   uint16_t len = htons(length);
+
   memcpy (p, &len, 2);
 
 #ifdef ZEBRA_HEADER_MARKER
@@ -324,27 +281,30 @@ char zebra_send_command (unsigned char command, char * options, int optlen) {
 	continue;
       }
       else {
+	olsr_printf (1, "(QUAGGA) Disconnected from zebra\n");
 	zebra.status &= ~STATUS_CONNECTED;
+	free (pnt);
 	return -1;
       }
     }
     p = p+ret;
   } while ((length -= ret));
-
+  free (pnt);
   return 0;
 }
 
 
 /* Creates a Route-Packet-Payload, needs address, netmask, nexthop, 
    distance, and a pointer of an size_t */
-static char* zebra_route_packet (struct ipv4_route r, ssize_t *optlen) {
+static unsigned char* zebra_route_packet (struct ipv4_route r, 
+                                          ssize_t *optlen) {
 
   int count;
 
-  char *cmdopt, *t;
+  unsigned char *cmdopt, *t;
   *optlen = 4; // first: type, flags, message, prefixlen
   *optlen += r.prefixlen / 8 + (r.prefixlen % 8 ? 1 : 0); // + prefix
-  if (r.message & ZAPI_MESSAGE_NEXTHOP)
+  if (r.message & ZAPI_MESSAGE_NEXTHOP) {
     if (r.nexthops->type == ZEBRA_NEXTHOP_IPV4 
 	|| r.nexthops->type == ZEBRA_NEXTHOP_IPV4_IFINDEX){
       *optlen += (sizeof r.nexthops->payload.v4 
@@ -352,6 +312,7 @@ static char* zebra_route_packet (struct ipv4_route r, ssize_t *optlen) {
     }
     else if (r.nexthops->type == 0) 
       *optlen += 5;
+  }
   if (r.message & ZAPI_MESSAGE_IFINDEX)
     *optlen += r.ind_num * sizeof *r.index + 1;
   if (r.message & ZAPI_MESSAGE_DISTANCE)
@@ -405,7 +366,7 @@ static char* zebra_route_packet (struct ipv4_route r, ssize_t *optlen) {
 /* adds a route to zebra-daemon */
 int zebra_add_v4_route (struct ipv4_route r) {
   
-  char *cmdopt;
+  unsigned char *cmdopt;
   ssize_t optlen;
   int retval;
 
@@ -420,7 +381,7 @@ int zebra_add_v4_route (struct ipv4_route r) {
 /* deletes a route from the zebra-daemon */
 int zebra_delete_v4_route (struct ipv4_route r) {
   
-  char *cmdopt;
+  unsigned char *cmdopt;
   ssize_t optlen;
   int retval;
 
@@ -435,22 +396,21 @@ int zebra_delete_v4_route (struct ipv4_route r) {
 
 
 /* Check wether there is data from zebra aviable */
-void zebra_check (void* foo) {
-  char *data, *f;
+void zebra_check (void* foo __attribute__((unused))) {
+  unsigned char *data, *f;
   ssize_t len, ret;
 
   if (!(zebra.status & STATUS_CONNECTED)) {
-    if (!zebra_reconnect()) return;
+    zebra_reconnect();
+    return;
   }
   data = try_read (&len);
   if (data) {
     f = data;
     do {
       ret = zebra_parse_packet (f, len);
-      if (!ret) {//something wired happened
-	puts ("DEBUG: IIIIIIIIIIRGS");
-	exit (EXIT_FAILURE);
-      }
+      if (!ret) // something wired happened
+	olsr_exit ("(QUAGGA) Zero message length??? ", EXIT_FAILURE);
       f += ret;
     } while ((f - data) < len);
     free (data);
@@ -460,8 +420,8 @@ void zebra_check (void* foo) {
 
 // tries to read a packet from zebra_socket
 // if there is something to read - make sure to read whole packages
-static char *try_read (ssize_t *len) {
-  char *buf = NULL;
+static unsigned char *try_read (ssize_t *len) {
+  unsigned char *buf = NULL;
   ssize_t ret = 0, bsize = 0;
   uint16_t length = 0, l = 0;
   int sockstate;
@@ -477,22 +437,16 @@ static char *try_read (ssize_t *len) {
       buf = my_realloc (buf, bsize, "Zebra try_read");
     }
     ret = read (zebra.sock, buf + l, bsize - l);
-    if (ret <= 0) {
-      if (errno == EAGAIN) {
-	errno = 0;
-      }
-      else {
-	olsr_printf(1, "OOPS, something realy wired happened:"
-		    "read returned %s\n", strerror(errno));
-	errno = 0;
-	zebra.status &= ~STATUS_CONNECTED;
-	return 0;
-      }
+    if (!ret) { // nothing more to read, packet is broken, discard!
       free (buf);
       return NULL;
     }
-    *len += ret;
+    if (ret < 0 && errno != EAGAIN) { // oops - we got disconnected
+      olsr_printf (1, "(QUAGGA) Disconnected from zebra\n");
+      zebra.status &= ~STATUS_CONNECTED;
+    }
 
+    *len += ret;
     while ((*len - l) > length) {
       l += length;
       memcpy (&length, buf + l, 2);
@@ -511,10 +465,13 @@ static char *try_read (ssize_t *len) {
 
 
 /* Parse a packet recived from zebra */
-int zebra_parse_packet (char *packet, ssize_t maxlen) {
+int zebra_parse_packet (unsigned char *packet, ssize_t maxlen) {
+
+  uint16_t command;
+  int skip;
 
   /* Array of functions */
-  int (*foo[ZEBRA_MESSAGE_MAX]) (char *, size_t) = {
+  int (*foo[ZEBRA_MESSAGE_MAX]) (unsigned char *, size_t) = {
     parse_interface_add,
     parse_interface_delete,
     parse_interface_address_add,
@@ -526,67 +483,79 @@ int zebra_parse_packet (char *packet, ssize_t maxlen) {
     parse_ipv6_route_add
   };
 
+  uint16_t length;
+  int ret;
+
 #ifdef MY_DEBUG
   puts ("DEBUG: zebra_parse_packet");
 #endif
-  uint16_t length;
-  
-  int ret;
+
   memcpy (&length, packet, 2);
   length = ntohs (length);
   
   if (maxlen < length) {
-    puts("Error: programmer is an idiot");
-    printf ("DEBUG: maxlen = %d, packet_length = %d\n", maxlen, length);
-    return maxlen;
+    olsr_printf (1, "(QUAGGA) maxlen = %lu, packet_length = %d\n", (unsigned long)maxlen, length);
+    olsr_exit ("(QUAGGA) programmer is an idiot", EXIT_FAILURE);
   }
 
-  if (packet[2] - 1 < ZEBRA_MESSAGE_MAX && foo[packet[2] - 1]) { 
-    if (!(ret = foo[packet[2] - 1] (packet + 3, length - 3))) 
+#ifdef ZEBRA_HEADER_MARKER
+  if (packet[2] == 255) { // found header marker!!
+    //packet[3] == ZSERV_VERSION: FIXME: HANDLE THIS!
+    memcpy (command, packet + 4, sizeof command); // two bytes command now!
+    command-- = ntohs (command); 
+    skip = 6;
+  }
+#else
+  command = packet[2] - 1;
+  skip = 3;
+#endif
+
+  if (command < ZEBRA_MESSAGE_MAX && foo[command]) { 
+    if (!(ret = foo[command] (packet + skip, length - skip))) 
       return length;
-    else printf ("DEBUG: Parse error: %d\n", ret);
+    else olsr_printf (1, "(QUAGGA) Parse error: %d\n", ret);
   }
   else
-    printf ("Unknown packet type: %d\n", packet[2]);
+    olsr_printf (1, "(QUAGGA) Unknown packet type: %d\n", packet[2]);
 
-  puts ("Quagga: RECIVED PACKET FROM ZEBRA THAT I CAN'T PARSE");
+  olsr_printf (1, "(Quagga) RECIVED PACKET FROM ZEBRA THAT I CAN'T PARSE");
 
   return length;
 }
 
 
-static int parse_interface_add (char *opt, size_t len) {
+static int parse_interface_add (unsigned char *opt __attribute__((unused)), size_t len __attribute__((unused))) {
   //todo
   return 0;
 }
 
 
-static int parse_interface_delete (char *opt, size_t len) {
+static int parse_interface_delete (unsigned char *opt __attribute__((unused)), size_t len __attribute__((unused))) {
   //todo
   return 0;
 }
 
 
-static int parse_interface_address_add (char *opt, size_t len) {
+static int parse_interface_address_add (unsigned char *opt __attribute__((unused)), size_t len __attribute__((unused))) {
   
   //todo
   return 0;
 }
 
-static int parse_interface_up (char *opt, size_t len) {
+static int parse_interface_up (unsigned char *opt __attribute__((unused)), size_t len __attribute__((unused))) {
   
   //todo
   return 0;
 }
 
-static int parse_interface_down (char *opt, size_t len) {
+static int parse_interface_down (unsigned char *opt __attribute__((unused)), size_t len __attribute__((unused))) {
   
   //todo
   return 0;
 }
 
 
-static int parse_interface_address_delete (char *opt, size_t  len) {
+static int parse_interface_address_delete (unsigned char *opt __attribute__((unused)), size_t  len __attribute__((unused))) {
   //todo
   return 0;
 }
@@ -594,7 +563,7 @@ static int parse_interface_address_delete (char *opt, size_t  len) {
 
 /* Parse an ipv4-route-packet recived from zebra
  */
-static int parse_ipv4_route (char *opt, size_t len, struct ipv4_route *r) {
+static int parse_ipv4_route (unsigned char *opt, size_t len, struct ipv4_route *r) {
   int c;
 
   if (len < 4) return -1;
@@ -653,22 +622,18 @@ static int parse_ipv4_route (char *opt, size_t len, struct ipv4_route *r) {
 }
 
 
-static int ipv4_route_add (char *opt, size_t len) {
+static int ipv4_route_add (unsigned char *opt, size_t len) {
 
   struct ipv4_route r;
   int f;
     
   f = parse_ipv4_route (opt, len, &r);
-  if (f < 0) {
-    printf ("parse-error: %d\n",f);
-    return f;
-  }
-  
-  add_hna4_route (r);
-  return 0;
+  if (f < 0) return f;
+
+  return add_hna4_route (r);
 }
 
-static int ipv4_route_delete (char *opt, size_t len) {
+static int ipv4_route_delete (unsigned char *opt, size_t len) {
   struct ipv4_route r;
   int f;
   
@@ -679,7 +644,7 @@ static int ipv4_route_delete (char *opt, size_t len) {
   
 }
 
-static int parse_ipv6_route_add (char *opt, size_t len) {
+static int parse_ipv6_route_add (unsigned char *opt __attribute__((unused)), size_t len __attribute__((unused))) {
   //todo
   return 0;
 }
@@ -757,6 +722,7 @@ static void free_ipv4_route (struct ipv4_route r) {
 
 }
 
+/*
 static uint8_t masktoprefixlen (uint32_t mask) {
   
   uint8_t prefixlen = 0;
@@ -768,6 +734,7 @@ static uint8_t masktoprefixlen (uint32_t mask) {
   return prefixlen;
   
 }
+*/
 
 int zebra_add_olsr_v4_route (struct rt_entry *r) {
   
@@ -777,14 +744,15 @@ int zebra_add_olsr_v4_route (struct rt_entry *r) {
   route.type = ZEBRA_ROUTE_OLSR; // OLSR
   route.message = ZAPI_MESSAGE_METRIC;
   route.flags = zebra.flags;
-  route.prefixlen = masktoprefixlen (r->rt_mask.v4);
-  route.prefix = r->rt_dst.v4;
-  if ((r->rt_router.v4 == r->rt_dst.v4 && route.prefixlen == 32)){
+  route.prefixlen =(r->rt_dst.prefix_len);
+  route.prefix = r->rt_dst.prefix.v4;
+  if ((r->rt_best->rtp_nexthop.gateway.v4 == r->rt_dst.prefix.v4 && 
+       route.prefixlen == 32)) {
     route.message |= ZAPI_MESSAGE_IFINDEX | ZAPI_MESSAGE_NEXTHOP;
     route.ind_num = 1;
     route.index = olsr_malloc (sizeof *route.index, 
 			       "zebra_add_olsr_v4_route");
-    *route.index = htonl(r->rt_if->if_index);
+    *route.index = htonl(r->rt_best->rtp_nexthop.iif_index);
     route.nexthops = olsr_malloc (sizeof route.nexthops->type +
 				  sizeof route.nexthops->payload,
 				  "zebra_add_olsr_v4_route");
@@ -799,10 +767,10 @@ int zebra_add_olsr_v4_route (struct rt_entry *r) {
 				   sizeof route.nexthops->payload), 
 				   "zebra_add_olsr_v4_route");
     route.nexthops->type = ZEBRA_NEXTHOP_IPV4;
-    route.nexthops->payload.v4 = r->rt_router.v4;
+    route.nexthops->payload.v4 = r->rt_best->rtp_nexthop.gateway.v4;
   }
 
-  route.metric = r->rt_metric;
+  route.metric = r->rt_best->rtp_metric.hops;
   route.metric = htonl(route.metric);
 
   if (zebra.distance) {
@@ -810,7 +778,6 @@ int zebra_add_olsr_v4_route (struct rt_entry *r) {
     route.distance = zebra.distance;
   }
 
-  add_v4_route_status (route);
   retval = zebra_add_v4_route(route);
   free_ipv4_route (route);
   return retval;
@@ -823,14 +790,15 @@ int zebra_del_olsr_v4_route (struct rt_entry *r) {
   route.type = ZEBRA_ROUTE_OLSR; // OLSR
   route.message = ZAPI_MESSAGE_METRIC;
   route.flags = zebra.flags;
-  route.prefixlen = masktoprefixlen (r->rt_mask.v4);
-  route.prefix = r->rt_dst.v4;
-  if ((r->rt_router.v4 == r->rt_dst.v4 && route.prefixlen == 32)){
+  route.prefixlen = r->rt_dst.prefix_len;
+  route.prefix = r->rt_dst.prefix.v4;
+  if ((r->rt_best->rtp_nexthop.gateway.v4 == r->rt_dst.prefix.v4 && 
+       route.prefixlen == 32)){
     route.message |= ZAPI_MESSAGE_IFINDEX;
     route.ind_num = 1;
     route.index = olsr_malloc (sizeof *route.index, 
 			       "zebra_add_olsr_v4_route");
-    *route.index = htonl (r->rt_if->if_index);
+    *route.index = htonl (r->rt_best->rtp_nexthop.iif_index);
     route.nexthops = olsr_malloc (sizeof route.nexthops->type +
 				  sizeof route.nexthops->payload,
 				  "zebra_add_olsr_v4_route");
@@ -845,9 +813,9 @@ int zebra_del_olsr_v4_route (struct rt_entry *r) {
 				   sizeof route.nexthops->payload), 
 				  "zebra_add_olsr_v4_route");
     route.nexthops->type = ZEBRA_NEXTHOP_IPV4;
-    route.nexthops->payload.v4 = r->rt_router.v4;
+    route.nexthops->payload.v4 = r->rt_best->rtp_nexthop.gateway.v4;
   }
-  route.metric = r->rt_metric;
+  route.metric = r->rt_best->rtp_metric.hops;
   route.metric = htonl (route.metric);
   
   if (zebra.distance) {
@@ -856,15 +824,22 @@ int zebra_del_olsr_v4_route (struct rt_entry *r) {
   }
 
   retval = zebra_delete_v4_route(route);
-  del_v4_route_status(route);
+
   free_ipv4_route (route);
   return retval;
 }
 
-void zebra_olsr_distance (char dist) {
+void zebra_olsr_distance (unsigned char dist) {
   zebra.distance = dist;
 }
 
 void zebra_olsr_localpref (void) {
   zebra.flags &= ZEBRA_FLAG_SELECTED;
 }
+
+void zebra_export_routes (unsigned char t) {
+  if (t)
+    zebra.options |= OPTION_EXPORT;
+  else
+    zebra.options &= ~OPTION_EXPORT;
+}
