@@ -36,7 +36,7 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: hashing.h,v 1.9 2007/01/31 12:36:50 bernd67 Exp $
+ * $Id: hashing.h,v 1.10 2007/08/02 14:47:26 bernd67 Exp $
  */
 
 
@@ -49,7 +49,7 @@
 #include "olsr_types.h"
 
 olsr_u32_t
-olsr_hashing(union olsr_ip_addr *);
+olsr_hashing(const union olsr_ip_addr *);
 
 
 #endif
