@@ -36,7 +36,7 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: ipc_frontend.h,v 1.13 2007/04/20 13:46:04 bernd67 Exp $
+ * $Id: ipc_frontend.h,v 1.15 2007/10/13 12:31:04 bernd67 Exp $
  */
 
 /*
@@ -98,8 +98,6 @@ struct ipc_net_msg
 };
 
 
-olsr_bool ipc_active;
-
 olsr_bool
 ipc_check_allowed_ip(union olsr_ip_addr *);
 
@@ -109,4 +107,7 @@ ipc_accept(int);
 void
 frontend_msgparser(union olsr_message *, struct interface *, union olsr_ip_addr *);
 
+int
+ipc_route_send_rtentry(union olsr_ip_addr *, union olsr_ip_addr *, int, int, const char *);
+
 #endif
